@@ -11,12 +11,13 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://genshi.edgewall.org/log/.
 
-import doctest
 import unittest
 
+from genshi.core import Attrs, QName
 from genshi.input import XML
 from genshi.path import Path, PathParser, PathSyntaxError, GenericStrategy, \
                         SingleStepStrategy, SimplePathStrategy
+from genshi.tests.test_utils import doctest_suite
 
 
 class FakePath(Path):
@@ -630,6 +631,25 @@ class PathTestCase(unittest.TestCase):
                               '/descendant::b/descendant::c', input=xml,
                               output='')
 
+    def test_attr_selection(self):
+        xml = XML('<root><foo bar="abc"></foo></root>')
+        path = Path('foo/@bar')
+        result = path.select(xml)
+        self.assertEqual(list(result), [
+            Attrs([(QName('bar'), u'abc')])
+        ])
+
+    def test_attr_selection_with_namespace(self):
+        xml = XML(
+            '<root xmlns:ns1="http://example.com">'
+            '<foo ns1:bar="abc"></foo>'
+            '</root>')
+        path = Path('foo/@ns1:bar')
+        result = path.select(xml, namespaces={'ns1': 'http://example.com'})
+        self.assertEqual(list(result), [
+            Attrs([(QName('http://example.com}bar'), u'abc')])
+        ])
+
     def _test_support(self, strategy_class, text):
         path = PathParser(text, None, -1).parse()[0]
         return strategy_class.supports(path)
@@ -688,7 +708,7 @@ class PathTestCase(unittest.TestCase):
 
 def suite():
     suite = unittest.TestSuite()
-    suite.addTest(doctest.DocTestSuite(Path.__module__))
+    suite.addTest(doctest_suite(Path.__module__))
     suite.addTest(unittest.makeSuite(PathTestCase, 'test'))
     return suite
 
